@@ -1,9 +1,5 @@
 #![cfg_attr(feature = "unstable", feature(unsize))]
 
-#[macro_use]
-extern crate coalesce;
-extern crate unreachable;
-
 mod util;
 mod array;
 mod vector;
